//
// Created by dell on 24-12-3.
//

#include "LDC1612.h"


void LDC1612_read_sensor_infomation(void)
{
    uint16_t device_id;
    uint8_t result;

    // 读取设备ID
    result = IIC_read_16bit(READ_DEVICE_ID, &device_id);

    if (result == 0) {
        printf("Device ID: 0x%04X\n", device_id);
    } else {
        printf("Failed to read Device ID\n");
    }

}





/**
  * @brief  This function configure the I2C peripheral & GPIO
  * @param[in]  none
  * @param[out] none
  * @retval None
  */
void LDC1612_I2CConfig(void) {
    rcu_periph_clock_enable(RCU_IR_GPIO);
    rcu_periph_clock_enable(RCU_I2C);

    gpio_af_set(I2C_SCL_PORT, I2C_GPIO_AF, I2C_SCL_PIN);
    gpio_af_set(I2C_SDA_PORT, I2C_GPIO_AF, I2C_SDA_PIN);

    gpio_mode_set(I2C_SCL_PORT, GPIO_MODE_AF, GPIO_PUPD_PULLUP, I2C_SCL_PIN);
    gpio_output_options_set(I2C_SCL_PORT, GPIO_OTYPE_OD, GPIO_OSPEED_50MHZ, I2C_SCL_PIN);

    gpio_mode_set(I2C_SDA_PORT, GPIO_MODE_AF, GPIO_PUPD_PULLUP, I2C_SDA_PIN);
    gpio_output_options_set(I2C_SDA_PORT, GPIO_OTYPE_OD, GPIO_OSPEED_50MHZ, I2C_SDA_PIN);

    i2c_clock_config(LDC_I2C, I2C_SPEED, I2C_DTCY_2);

    i2c_mode_addr_config(LDC_I2C, I2C_I2CMODE_ENABLE, I2C_ADDFORMAT_7BITS, LDC1612_ADDR);
    i2c_enable(LDC_I2C);
    i2c_ack_config(LDC_I2C, I2C_ACK_ENABLE);
}


/**
 * @brief  向I2C设备寄存器写入数据
 * @param  device_address: 设备地址
 * @param  data: 要写入的数据
 * @retval int: 0表示成功，-1表示超时
 */
int i2c_write_reg(uint8_t device_address, uint8_t *data)
{
    uint32_t timeout = 0;
    printf("debug tag- 0\r\n");
    // 确保 I2C 总线空闲
    while (i2c_flag_get(LDC_I2C, I2C_FLAG_I2CBSY) && (timeout < I2C_TIME_OUT))
        timeout++;
    if (timeout >= I2C_TIME_OUT) {
        printf("debug tag- 1-\r\n");
        return -1; // 超时返回错误
    }
    i2c_start_on_bus(LDC_I2C);
    timeout = 0;
    printf("debug tag- 1\r\n");

    // 等待起始条件发送完成
    while (!i2c_flag_get(LDC_I2C, I2C_FLAG_SBSEND) && (timeout < I2C_TIME_OUT))
        timeout++;
    if (timeout >= I2C_TIME_OUT) {
        return -1; // 超时返回错误
    }
    i2c_master_addressing(LDC_I2C, device_address, I2C_TRANSMITTER);
    timeout = 0;
    printf("debug tag- 2\r\n");

    // 等待地址发送完成
    while (!i2c_flag_get(LDC_I2C, I2C_FLAG_ADDSEND) && (timeout < I2C_TIME_OUT))
        timeout++;
    if (timeout >= I2C_TIME_OUT) {
        printf("debug tag- 3-\r\n");
        return -1; // 超时返回错误
    }
    i2c_flag_clear(LDC_I2C, I2C_FLAG_ADDSEND);
    timeout = 0;
    printf("debug tag- 3\r\n");

    // 发送寄存器地址和数据
    for (int i = 0; i < 3; i++) {
        while (!i2c_flag_get(LDC_I2C, I2C_FLAG_TBE) && (timeout < I2C_TIME_OUT))
            timeout++;
        if (timeout >= I2C_TIME_OUT) {
            return -1; // 超时返回错误
        }
        i2c_data_transmit(LDC_I2C, data[i]);
        printf("iic_value:%x\r\n", data[i]);
        timeout = 0;
    }
    timeout = 0;

    // 生成停止条件
    while (!i2c_flag_get(LDC_I2C, I2C_FLAG_BTC) && (timeout < I2C_TIME_OUT))
        timeout++;
    if (timeout >= I2C_TIME_OUT) {
        return -1; // 超时返回错误
    }
    i2c_stop_on_bus(LDC_I2C);
    timeout = 0;

    // 确保停止条件已发送完成
    while (i2c_flag_get(LDC_I2C, I2C_FLAG_I2CBSY) && (timeout < I2C_TIME_OUT))
        timeout++;
    if (timeout >= I2C_TIME_OUT) {
        return -1; // 超时返回错误
    }

    printf("test info\r\n");
    return 0;
}

uint8_t eeprom_byte_write_timeout(uint8_t device_address, uint8_t *data)
{
    uint8_t state = I2C_START;
    uint16_t timeout = 0;
    uint8_t i2c_timeout_flag = 0;

    /* enable acknowledge */
    i2c_ack_config(LDC_I2C, I2C_ACK_ENABLE);
    while(!(i2c_timeout_flag)) {
        switch(state) {
        case I2C_START:
            /* i2c master sends start signal only when the bus is idle */
            while(i2c_flag_get(LDC_I2C, I2C_FLAG_I2CBSY) && (timeout < I2C_TIME_OUT)) {
                timeout++;
            }
            if(timeout < I2C_TIME_OUT) {
                i2c_start_on_bus(LDC_I2C);
                timeout = 0;
                state = I2C_SEND_ADDRESS;
            } else {
                timeout = 0;
                state   = I2C_START;
                printf("i2c bus is busy in WRITE BYTE!\n");
            }
            break;
        case I2C_SEND_ADDRESS:
            /* i2c master sends START signal successfully */
            while((!i2c_flag_get(LDC_I2C, I2C_FLAG_SBSEND)) && (timeout < I2C_TIME_OUT)) {
                timeout++;
            }
            if(timeout < I2C_TIME_OUT) {
                i2c_master_addressing(LDC_I2C, device_address, I2C_TRANSMITTER);
                timeout = 0;
                state = I2C_CLEAR_ADDRESS_FLAG;
            } else {
                timeout = 0;
                state = I2C_START;
                printf("i2c master sends start signal timeout in WRITE BYTE!\n");
            }
            break;
        case I2C_CLEAR_ADDRESS_FLAG:
            /* address flag set means i2c slave sends ACK */
            while((!i2c_flag_get(LDC_I2C, I2C_FLAG_ADDSEND)) && (timeout < I2C_TIME_OUT)) {
                timeout++;
            }
            if(timeout < I2C_TIME_OUT) {
                i2c_flag_clear(LDC_I2C, I2C_FLAG_ADDSEND);
                timeout = 0;
                state = I2C_TRANSMIT_DATA;
            } else {
                timeout = 0;
                state = I2C_START;
                printf("i2c master clears address flag timeout in WRITE BYTE!\n");
            }
            break;
        case I2C_TRANSMIT_DATA:
            /* wait until the transmit data buffer is empty */
            while((!i2c_flag_get(LDC_I2C, I2C_FLAG_TBE)) && (timeout < I2C_TIME_OUT)) {
                timeout++;
            }
            if(timeout < I2C_TIME_OUT) {
                /* send the EEPROM's internal address to write to : only one byte address */
                i2c_data_transmit(LDC_I2C, data[0]);
                timeout = 0;
            } else {
                timeout = 0;
                state = I2C_START;
                printf("i2c master sends data timeout in WRITE BYTE!\n");
            }

            /* wait until BTC bit is set */
            while((!i2c_flag_get(LDC_I2C, I2C_FLAG_BTC)) && (timeout < I2C_TIME_OUT)) {
                timeout++;
            }
            if(timeout < I2C_TIME_OUT) {
                /* send the EEPROM's internal address to write to : only one byte address */
                i2c_data_transmit(LDC_I2C, data[1]);
                timeout = 0;
            } else {
                timeout = 0;
                state = I2C_START;
                printf("i2c master sends data timeout in WRITE 1 BYTE!\n");
            }
            /* wait until BTC bit is set */
            while((!i2c_flag_get(LDC_I2C, I2C_FLAG_BTC)) && (timeout < I2C_TIME_OUT)) {
                timeout++;
            }
            if(timeout < I2C_TIME_OUT) {
                /* send the EEPROM's internal address to write to : only one byte address */
                i2c_data_transmit(LDC_I2C, data[2]);
                timeout = 0;
            } else {
                timeout = 0;
                state = I2C_START;
                printf("i2c master sends data timeout in WRITE 2 BYTE!\n");
            }
            /* wait until BTC bit is set */
            while((!i2c_flag_get(LDC_I2C, I2C_FLAG_BTC)) && (timeout < I2C_TIME_OUT)) {
                timeout++;
            }
            if(timeout < I2C_TIME_OUT) {
                state = I2C_STOP;
                timeout = 0;
            } else {
                timeout = 0;
                state = I2C_START;
                printf("i2c master sends data timeout in WRITE BYTE!\n");
            }
            break;
        case I2C_STOP:
            /* send a stop condition to I2C bus */
            i2c_stop_on_bus(LDC_I2C);
            /* i2c master sends STOP signal successfully */
            while((I2C_CTL0(LDC_I2C) & I2C_CTL0_STOP) && (timeout < I2C_TIME_OUT)) {
                timeout++;
            }
            if(timeout < I2C_TIME_OUT) {
                timeout = 0;
                state = I2C_END;
                i2c_timeout_flag = I2C_OK;
            } else {
                timeout = 0;
                state = I2C_START;
                printf("i2c master sends stop signal timeout in WRITE BYTE!\n");
            }
            break;
        default:
            state = I2C_START;
            i2c_timeout_flag = I2C_OK;
            timeout = 0;
            printf("i2c master sends start signal in WRITE BYTE.\n");
            break;
        }
    }
    return I2C_END;
}

void LDC1612_Init(void) {
    uint8_t RCOUNT0_ALL[3]={RCOUNT0_ADDR,0x05,0x36};//csdn
    uint8_t SETTLECOUNT0_ALL[3]={SETTLECOUNT0_ADDR,0x00,0x0a};
    uint8_t CLOCK_DIVIDERS0_ALL[3]={CLOCK_DIVIDERS0_ADDR,0x10,0x02};
    uint8_t ERROR_CONFIG_ALL[3]={ERROR_CONFIG_ADDR,0x00,0x00};
    uint8_t MUX_CONFIG_ALL[3]={MUX_CONFIG_ADDR,0x82,0x0c};
    uint8_t DRIVE_CURRENT0_ALL[3]={DRIVE_CURRENT0_ADDR,0x90,0x00};
    uint8_t CONFIG_ALL[3]={CONFIG_ADDR,0x14,0x01};//csdn

    // LDC1612_I2CConfig();

    // int ret = i2c_write_reg(LDC1612_ADDR, RCOUNT0_ALL);
    //
    // printf("ret:%d\r\n",ret);
    i2c_write_reg(LDC1612_ADDR, RCOUNT0_ALL);
    // eeprom_byte_write_timeout(LDC1612_ADDR, SETTLECOUNT0_ALL);
    delay_ms(100);
    i2c_write_reg(LDC1612_ADDR, SETTLECOUNT0_ALL);
    delay_ms(100);
    // i2c_write_reg(LDC1612_ADDR, CLOCK_DIVIDERS0_ALL);
    // delay_ms(100);
    // i2c_write_reg(LDC1612_ADDR, ERROR_CONFIG_ALL);
    // delay_ms(100);
    // i2c_write_reg(LDC1612_ADDR, MUX_CONFIG_ALL);
    // delay_ms(100);
    // i2c_write_reg(LDC1612_ADDR, DRIVE_CURRENT0_ALL);
    // delay_ms(100);
    // i2c_write_reg(LDC1612_ADDR, CONFIG_ALL);
}

/**
 * @brief  从I2C设备寄存器读取数据
 * @param  device_address: 设备地址
 * @param  reg_address: 寄存器地址
 * @param  data: 存储读取数据的缓冲区
 * @param  length: 要读取的数据长度
 * @retval int: 0表示成功，-1表示超时
 */
int LDC1612_read_reg(uint8_t device_address, uint8_t reg_address, uint8_t *data, uint16_t length)
{
    uint32_t timeout = 0;

    i2c_ack_config(LDC_I2C, I2C_ACK_ENABLE);

    // 判断总线是否忙
    while (i2c_flag_get(LDC_I2C, I2C_FLAG_I2CBSY) && (timeout < I2C_TIME_OUT))
        timeout++;
    if (timeout >= I2C_TIME_OUT) {
        return -1; // 超时返回错误
    }
    i2c_start_on_bus(LDC_I2C); //发送开始信号
    timeout = 0;

    // 等待起始条件发送完成
    while (!i2c_flag_get(LDC_I2C, I2C_FLAG_SBSEND) && (timeout < I2C_TIME_OUT))
        timeout++;
    if (timeout >= I2C_TIME_OUT) {
        return -1; // 超时返回错误
    }
    i2c_master_addressing(LDC_I2C, device_address, I2C_TRANSMITTER); //设置为发送模式
    timeout = 0;

    // 等待地址发送完成
    while (!i2c_flag_get(LDC_I2C, I2C_FLAG_ADDSEND) && (timeout < I2C_TIME_OUT))
        timeout++;
    if (timeout >= I2C_TIME_OUT) {
        return -1; // 超时返回错误
    }
    i2c_flag_clear(LDC_I2C, I2C_FLAG_ADDSEND);
    timeout = 0;

    // 开始接收数据
    while (!i2c_flag_get(LDC_I2C, I2C_FLAG_TBE) && (timeout < I2C_TIME_OUT))
        timeout++;
    if (timeout >= I2C_TIME_OUT) {
        return -1; // 超时返回错误
    }
    i2c_data_transmit(LDC_I2C, reg_address); //发送寄存器地址
    timeout = 0;

    // 生成重复起始条件
    while (!i2c_flag_get(LDC_I2C, I2C_FLAG_BTC) && (timeout < I2C_TIME_OUT))
        timeout++;
    if (timeout >= I2C_TIME_OUT) {
        return -1; // 超时返回错误
    }
    i2c_start_on_bus(LDC_I2C);
    timeout = 0;

    // 等待重复起始条件发送完成
    while (!i2c_flag_get(LDC_I2C, I2C_FLAG_SBSEND) && (timeout < I2C_TIME_OUT))
        timeout++;
    if (timeout >= I2C_TIME_OUT) {
        return -1; // 超时返回错误
    }
    i2c_master_addressing(LDC_I2C, device_address, I2C_RECEIVER);
    timeout = 0;

    // 等待地址发送完成
    while (!i2c_flag_get(LDC_I2C, I2C_FLAG_ADDSEND) && (timeout < I2C_TIME_OUT))
        timeout++;
    if (timeout >= I2C_TIME_OUT) {
        return -1; // 超时返回错误
    }
    i2c_flag_clear(LDC_I2C, I2C_FLAG_ADDSEND);
    timeout = 0;

// 读取数据
for (uint16_t i = 0; i < length; i++) {
    while (!i2c_flag_get(LDC_I2C, I2C_FLAG_RBNE) && (timeout < I2C_TIME_OUT))
        timeout++;
    if (timeout >= I2C_TIME_OUT) {
        return -1; // 超时返回错误
    }
    if (i == length - 1) {
        i2c_ack_config(LDC_I2C, I2C_ACK_DISABLE); // 禁用ACK
        i2c_stop_on_bus(LDC_I2C); // 生成停止条件
    }
    data[i] = i2c_data_receive(LDC_I2C);
    timeout = 0;
}
i2c_ack_config(LDC_I2C, I2C_ACK_ENABLE); // 重新启用ACK

    // 生成停止条件
    i2c_stop_on_bus(LDC_I2C);
    timeout = 0;

    return 0;
}

void I2C_Scan(void) {
    uint32_t timeout;
    uint8_t address;
    int found_devices = 0;

    printf("Scanning I2C bus...\r\n");

    for (address = 1; address < 127; address++) {
        timeout = 0;

        // 生成起始条件
        while (i2c_flag_get(LDC_I2C, I2C_FLAG_I2CBSY) && (timeout < I2C_TIME_OUT))
            timeout++;
        if (timeout >= I2C_TIME_OUT) {
            continue; // 超时，跳过该地址
        }
        i2c_start_on_bus(LDC_I2C);
        timeout = 0;

        // 等待起始条件发送完成
        while (!i2c_flag_get(LDC_I2C, I2C_FLAG_SBSEND) && (timeout < I2C_TIME_OUT))
            timeout++;
        if (timeout >= I2C_TIME_OUT) {
            continue; // 超时，跳过该地址
        }
        i2c_master_addressing(LDC_I2C, address << 1, I2C_TRANSMITTER);
        timeout = 0;

        // 等待地址发送完成
        while (!i2c_flag_get(LDC_I2C, I2C_FLAG_ADDSEND) && (timeout < I2C_TIME_OUT))
            timeout++;
        if (timeout < I2C_TIME_OUT) {
            i2c_flag_clear(LDC_I2C, I2C_FLAG_ADDSEND);
            printf("Found device at 0x%02X\r\n", address);
            found_devices++;
        }

        // 生成停止条件
        i2c_stop_on_bus(LDC_I2C);
        timeout = 0;

        while (i2c_flag_get(LDC_I2C, I2C_FLAG_STPDET) && (timeout < I2C_TIME_OUT))
            timeout++;
    }

    if (found_devices == 0) {
        printf("No I2C devices found.\r\n");
    } else {
        printf("Total %d I2C devices found.\r\n", found_devices);
    }
}



/**
 * @brief  读取16位I2C数据，带有超时机制
 * @param  reg: 要读取的寄存器地址
 * @param  value: 存储读取数据的指针
 * @retval 0: 成功，1: 超时
 */
uint8_t IIC_read_16bit(uint8_t reg, uint16_t *value) {
    uint32_t timeout = 0;

    // 发送启动信号
    i2c_start_on_bus(I2C0);
    timeout = 0;
    while (!i2c_flag_get(I2C0, I2C_FLAG_SBSEND)) {
        if ((timeout++) > I2C_TIME_OUT) return 1;  // 超时
    }

    // 发送从设备地址和写信号
    i2c_master_addressing(I2C0, LDC1612_ADDR, I2C_TRANSMITTER);
    timeout = 0;
    while (!i2c_flag_get(I2C0, I2C_FLAG_ADDSEND)) {
        if ((timeout++) > I2C_TIME_OUT) return 1;  // 超时
    }
    i2c_flag_clear(I2C0, I2C_FLAG_ADDSEND);

    // 发送寄存器地址
    i2c_data_transmit(I2C0, reg);
    timeout = 0;
    while (!i2c_flag_get(I2C0, I2C_FLAG_TBE)) {
        if ((timeout++) > I2C_TIME_OUT) return 1;  // 超时
    }

    // 重新启动信号
    i2c_start_on_bus(I2C0);
    timeout = 0;
    while (!i2c_flag_get(I2C0, I2C_FLAG_SBSEND)) {
        if ((timeout++) > I2C_TIME_OUT) return 1;  // 超时
    }

    // 发送从设备地址和读信号
    i2c_master_addressing(I2C0, LDC1612_ADDR, I2C_RECEIVER);
    timeout = 0;
    while (!i2c_flag_get(I2C0, I2C_FLAG_ADDSEND)) {
        if ((timeout++) > I2C_TIME_OUT) return 1;  // 超时
    }
    i2c_flag_clear(I2C0, I2C_FLAG_ADDSEND);

    // 读取数据
    timeout = 0;
    while (!i2c_flag_get(I2C0, I2C_FLAG_RBNE)) {
        if ((timeout++) > I2C_TIME_OUT) return 1;  // 超时
    }
    *value = i2c_data_receive(I2C0) << 8;  // 读取高8位

    timeout = 0;
    while (!i2c_flag_get(I2C0, I2C_FLAG_RBNE)) {
        if ((timeout++) > I2C_TIME_OUT) return 1;  // 超时
    }
    *value |= i2c_data_receive(I2C0);  // 读取低8位

    // 发送停止信号
    i2c_stop_on_bus(I2C0);
    timeout = 0;
    while (I2C_CTL0(I2C0) & I2C_CTL0_STOP) {
        if ((timeout++) > I2C_TIME_OUT) return 1;  // 超时
    }

    return 0;  // 成功
}


int LDC_getID(void) {
    uint8_t data[2] = {0};
    uint16_t timeout = 0;

    i2c_ack_config(I2C0, I2C_ACK_ENABLE);

    while (i2c_flag_get(I2C0, I2C_FLAG_I2CBSY) && (timeout < I2C_TIME_OUT)) //判断IIC总线是否忙，发送起始信号
        timeout++;
    if (timeout < I2C_TIME_OUT) {
        i2c_start_on_bus(I2C0);
        timeout = 0;
    } else {
        printf("err\r\n");
        return -4101; // 超时返回错误
    }

    while (!i2c_flag_get(I2C0, I2C_FLAG_SBSEND) && (timeout < I2C_TIME_OUT)) //判断起始位是否发送，设置sensor地址并设置为写
        timeout++;
    if (timeout < I2C_TIME_OUT) {
        i2c_master_addressing(I2C0, (0x2B << 1), I2C_TRANSMITTER);
        timeout = 0;
    } else {
        return -4102; // 超时返回错误
    }

    while (!i2c_flag_get(I2C0, I2C_FLAG_ADDSEND) && (timeout < I2C_TIME_OUT))
        timeout++;
    if (timeout < I2C_TIME_OUT) {
        i2c_flag_clear(I2C0, I2C_FLAG_ADDSEND);
        timeout = 0;
    } else {
        return -4103; // 超时返回错误
    }

    while (!i2c_flag_get(I2C0, I2C_FLAG_TBE) && (timeout < I2C_TIME_OUT)) //判断地址是否发送完成，然后发送寄存器地址
        timeout++;
    if (timeout < I2C_TIME_OUT) {
        i2c_data_transmit(I2C0, 0x7f);
        timeout = 0;
        // i2c_start_on_bus(IR_I2C);
    } else {
        return -4104; // 超时返回错误
    }

    while (i2c_flag_get(I2C0, I2C_FLAG_BTC) && (timeout < I2C_TIME_OUT)) //判断发送缓冲器是否为空，为空后（发送完毕）重新发送开始信号
        timeout++;
    if (timeout < I2C_TIME_OUT) {
        i2c_start_on_bus(I2C0);
        timeout = 0;
    } else {
        return -4105; // 超时返回错误
    }

    while (!i2c_flag_get(I2C0, I2C_FLAG_SBSEND) && (timeout < I2C_TIME_OUT)) {
        timeout++;
    }
    if (timeout < I2C_TIME_OUT) {
        i2c_master_addressing(I2C0, (0x2B << 1), I2C_RECEIVER);
        timeout = 0;
    } else {
        return -4106; // 超时返回错误
    }

    while (!i2c_flag_get(I2C0, I2C_FLAG_ADDSEND) && (timeout < I2C_TIME_OUT))
        timeout++;
    if (timeout < I2C_TIME_OUT) {
        i2c_flag_clear(I2C0, I2C_FLAG_ADDSEND);
        timeout = 0;
    } else {
        return -4107; // 超时返回错误
    }

    // 读取第一个字节的数据
    while (!i2c_flag_get(I2C0, I2C_FLAG_RBNE) && (timeout < I2C_TIME_OUT)) {
        timeout++;
    }
    if (timeout < I2C_TIME_OUT) {
        data[0] = i2c_data_receive(I2C0);
        timeout = 0;
    } else {
        return -4108; // 超时返回错误
    }
    i2c_ack_config(I2C0, I2C_ACK_DISABLE);

    // 读取第二个字节的数据
    while (!i2c_flag_get(I2C0, I2C_FLAG_RBNE) && (timeout < I2C_TIME_OUT)) {
        timeout++;
    }
    if (timeout < I2C_TIME_OUT) {
        data[1] = i2c_data_receive(I2C0);
        timeout = 0;
    } else {
        return -4109; // 超时返回错误
    }

    i2c_stop_on_bus(I2C0);

    printf("device id:%x %x\r\n",data[0],data[1]);

    return 0;
}